@@ -26,11 +26,11 @@
 // Qt
 #include <QDebug>
 #include <QtCore/QDir>
-#include <QtGui/QPushButton>
-#include <QtGui/QLayout>
-#include <QtGui/QSplitter>
+#include <QPushButton>
+#include <QLayout>
+#include <QSplitter>
 #include <QtCore/QStringList>
-#include <QtGui/QMenu>
+#include <QMenu>
 
 // KDE
 #include <klocale.h>
@@ -40,6 +40,7 @@
 #include <kdesktopfile.h>
 #include <kiconloader.h>
 #include <kicondialog.h>
+#include <kicon.h>
 #include <kmessagebox.h>
 #include <kinputdialog.h>
 #include <konq_events.h>
